==> org-lsst-ccs-subsystem-vacuum-main-1.2.4-sources.jar: org/lsst/ccs/subsystem/vacuum/VacPlutoDevice.java <==
package org.lsst.ccs.subsystem.vacuum;

import org.lsst.ccs.subsystem.common.devices.pluto.PlutoDevice;
import org.lsst.ccs.subsystem.vacuum.constants.Buttons;
import org.lsst.ccs.subsystem.vacuum.constants.Conditions;
import org.lsst.ccs.subsystem.vacuum.constants.Devices;
import org.lsst.ccs.subsystem.vacuum.constants.Latches;

/**
 *  Handles the vacuum system Pluto PLC.
 *
 *  @author Owen Saxton
 */
public class VacPlutoDevice extends PlutoDevice implements SwitchDevice {

    /**
     *  Constants.
     */
    public static final int
        SW_OPEN_VHX00     = 0,
        SW_OPEN_VCR00     = 1,
        SW_OPEN_VCR01     = 2,
        SW_OPEN_VCR02     = 3,
        SW_OPEN_VCR03     = 4,
        SW_OPEN_VCR04     = 5,
        SW_BLOCK_CR_VAC   = 6,
        SW_BLOCK_HX_VAC   = 7,
        SW_BLOCK_CR_TURBO = 8,
        SW_BLOCK_HX_TURBO = 9,
        NUM_SWITCHES      = 10;

    private static final int
        NUM_AREAS    = 19,
        SWDI_ON_BIT = 0,
        SWDI_OFF_BIT = 1,
        SWDI_READ_AREA = 2,
        SWDI_READ_BIT = 3,
        LTDI_RESET_BIT = 0,
        LTDI_READ_AREA = 1,
        LTDI_READ_BIT = 2,
        LTDI_PEND_AREA = 3,
        LTDI_PEND_BIT = 4,
        CNDI_READ_AREA = 0,
        CNDI_READ_BIT = 1;

    /**
     *  Private lookup maps, etc.
     */
    private static final int[][] switches = new int[NUM_SWITCHES][];
    static {
        switches[SW_OPEN_VCR00] = new int[]{7, 6, 15, 0};   // ON bit, OFF bit, read area, read bit 
        switches[SW_OPEN_VCR01] = new int[]{12, 13, 12, 0};
        switches[SW_OPEN_VCR02] = new int[]{14, 15, 12, 3};
        switches[SW_OPEN_VCR03] = new int[]{16, 17, 13, 0};
        switches[SW_OPEN_VCR04] = new int[]{18, 19, 13, 3};
        switches[SW_OPEN_VHX00] = new int[]{3, 2, 15, 2};
        switches[SW_BLOCK_CR_VAC] = new int[]{21, 1, 14, 3};
        switches[SW_BLOCK_HX_VAC] = new int[]{20, 0, 14, 0};
        switches[SW_BLOCK_CR_TURBO] = new int[]{23, 11, 11, 3};
        switches[SW_BLOCK_HX_TURBO] = new int[]{22, 10, 11, 0};
    }
    private static final int[] buttons = new int[Buttons.NUM_BUTTONS];
    static {
        buttons[Buttons.BTN_BLOCK_HVSTAT] = 20;
        buttons[Buttons.BTN_BLOCK_CVSTAT] = 21;
        buttons[Buttons.BTN_BLOCK_HVPUMP] = 22;
        buttons[Buttons.BTN_BLOCK_CVPUMP] = 23;
    }
    private static final int[][] latches = new int[Latches.NUM_LATCHES][];
    static {
        latches[Latches.LATCH_CR_VACUUM] = new int[]{1, 14, 3, 18, 0};   // Reset bit, on area, on bit,
        latches[Latches.LATCH_CR_GATE_AO] = new int[]{9, 15, 4, 18, 2};  //    ind area, ind bit
        latches[Latches.LATCH_CR_GATE_NFC] = new int[]{8, 10, 3, 18, 1};
        latches[Latches.LATCH_CR_PUMP] = new int[]{11, 11, 3, 18, 3};
        latches[Latches.LATCH_HX_VACUUM] = new int[]{0, 14, 0, 18, 4};
        latches[Latches.LATCH_HX_GATE_AO] = new int[]{5, 15, 5, 18, 6};
        latches[Latches.LATCH_HX_GATE_NFC] = new int[]{4, 9, 3, 18, 5};
        latches[Latches.LATCH_HX_PUMP] = new int[]{10, 11, 0, 18, 7};
    }
    private static final int[][] conditions = new int[Conditions.NUM_CONDITIONS][];
    static {
        conditions[Conditions.COND_CR_FORELINE_VAC] = new int[]{11, 6};  // Area, bit
        conditions[Conditions.COND_CR_TRB_PRESS_10] = new int[]{16, 5};
        conditions[Conditions.COND_CR_TRB_PUMP_OFF] = new int[]{16, 6};
        conditions[Conditions.COND_CR_VACUUM_001] = new int[]{17, 5};
        conditions[Conditions.COND_CR_VACUUM_01] = new int[]{17, 4};
        conditions[Conditions.COND_HX_FORELINE_VAC] = new int[]{11, 7};
        conditions[Conditions.COND_HX_TRB_PRESS_10] = new int[]{16, 3};
        conditions[Conditions.COND_HX_TURBO_OFF] = new int[]{16, 4};
        conditions[Conditions.COND_HX_VACUUM_001] = new int[]{17, 7};
        conditions[Conditions.COND_HX_VACUUM_01] = new int[]{17, 6};
    }


    /**
     *   Constructor.
     */
    public VacPlutoDevice()
    {
        super(NUM_AREAS);
    }


    /**
     *  Performs basic initialization.
     */
    @Override
    protected void initDevice()
    {
        super.initDevice();
    }


    /**
     *  Gets the switch device ID.
     * 
     *  @return  The device ID
     */
    @Override
    public int getSwitchDevice()
    {
        return Devices.DEVC_PLUTO;
    }


    /**
     *  Sets a switch on or off.
     *
     *  For the vacuum Pluto, this is implemented as a pair of push buttons,
     *  one for on, one for off.
     *
     *  @param  sw  The switch number.
     *  @param  on  The on state to set: true or false
     */
    @Override
    public void setSwitch(int sw, boolean on)
    {
        int bitNum = switches[sw][on ? SWDI_ON_BIT : SWDI_OFF_BIT];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets the on state of a switch.
     *
     *  The state is not the state of the bit that was toggled, but is read back
     *  either directly from the controlled hardware, or from the PLC output line.
     *
     *  @param  sw  The switch number.
     *  @return  Whether the switch is on
     */
    @Override
    public Boolean isSwitchOn(int sw)
    {
        int[] swData = switches[sw];
        Integer value = readAddBit(swData[SWDI_READ_AREA], swData[SWDI_READ_BIT] + 16);
        return value != null ? value != 0 : null;
    }


    /**
     *  Presses a button.
     * 
     *  @param  btn  The button number
     */
    public void pressButton(int btn)
    {
        int bitNum = buttons[btn];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets whether a latched condition is active.
     * 
     *  @param  cond  The condition number
     *  @return  Whether active - indicated by the bit being 0
     */
    public Boolean isLatchActive(int cond)
    {
        int[] condData = latches[cond];
        Integer value = readAddBit(condData[LTDI_READ_AREA], condData[LTDI_READ_BIT] + 16);
        return value != null ? value == 0 : null;
    }


    /**
     *  Gets whether a latched condition is latched.
     * 
     *  @param  cond  The condition number
     *  @return  Whether latched - indicated by the bit being 1
     */
    public Boolean isLatchLatched(int cond)
    {
        int[] condData = latches[cond];
        Integer value = readAddBit(condData[LTDI_PEND_AREA], condData[LTDI_PEND_BIT] + 16);
        return value != null ? value != 0 : null;
    }


    /**
     *  Clears a latched condition.
     * 
     *  @param  cond  The condition number
     */
    public void clearLatch(int cond)
    {
        int bitNum = latches[cond][LTDI_RESET_BIT];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets whether a condition is active.
     * 
     *  @param  cond  The condition number
     *  @return  Whether active - indicated by the bit being 1
     */
    public Boolean isConditionActive(int cond)
    {
        int[] condData = conditions[cond];
        Integer value = readAddBit(condData[CNDI_READ_AREA], condData[CNDI_READ_BIT] + 16);
        return value != null ? value != 0 : null;
    }

}

==> org-lsst-ccs-subsystem-vacuum-main-1.2.22-20250303.171723-7-sources.jar: org/lsst/ccs/subsystem/vacuum/VacPlutoDevice.java <==
package org.lsst.ccs.subsystem.vacuum;

import java.util.HashMap;
import java.util.Map;
import org.lsst.ccs.subsystem.common.devices.pluto.PlutoDevice;
import org.lsst.ccs.subsystem.vacuum.constants.Buttons;
import org.lsst.ccs.subsystem.vacuum.constants.Conditions;
import org.lsst.ccs.subsystem.vacuum.constants.DeviceState;
import org.lsst.ccs.subsystem.vacuum.constants.Devices;
import org.lsst.ccs.subsystem.vacuum.constants.Latches;

/**
 *  Handles the vacuum system Pluto PLC.
 *
 *  @author Owen Saxton
 */
public class VacPlutoDevice extends PlutoDevice implements SwitchDevice {

    /**
     *  Constants.
     */
    public static final int
        SW_OPEN_VHX00     = 0,
        SW_OPEN_VCR00     = 1,
        SW_OPEN_VCR01     = 2,
        SW_OPEN_VCR02     = 3,
        SW_OPEN_VCR03     = 4,
        SW_OPEN_VCR04     = 5,
        SW_BLOCK_CR_VAC   = 6,
        SW_BLOCK_HX_VAC   = 7,
        SW_BLOCK_CR_TURBO = 8,
        SW_BLOCK_HX_TURBO = 9,
        NUM_SWITCHES      = 10;

    private static final int
        NUM_AREAS    = 19,
        SWDI_ON_BIT = 0,
        SWDI_OFF_BIT = 1,
        SWDI_READ_AREA = 2,
        SWDI_READ_BIT = 3,
        SWSI_READ_AREA = 0,
        SWSI_OPEN_BIT = 1,
        SWSI_SHUT_BIT = 2,
        LTDI_RESET_BIT = 0,
        LTDI_READ_AREA = 1,
        LTDI_READ_BIT = 2,
        LTDI_PEND_AREA = 3,
        LTDI_PEND_BIT = 4,
        CNDI_READ_AREA = 0,
        CNDI_READ_BIT = 1;

    /**
     *  Private lookup maps, etc.
     */
    private static final int[][] switches = new int[NUM_SWITCHES][];
    static {
        switches[SW_OPEN_VCR00] = new int[]{7, 6, 10, 0};   // ON bit, OFF bit, read area, read bit 
        switches[SW_OPEN_VCR01] = new int[]{12, 13, 12, 0};
        switches[SW_OPEN_VCR02] = new int[]{14, 15, 12, 3};
        switches[SW_OPEN_VCR03] = new int[]{16, 17, 13, 0};
        switches[SW_OPEN_VCR04] = new int[]{18, 19, 13, 3};
        switches[SW_OPEN_VHX00] = new int[]{3, 2, 9, 0};
        switches[SW_BLOCK_CR_VAC] = new int[]{21, 1, 14, 3};
        switches[SW_BLOCK_HX_VAC] = new int[]{20, 0, 14, 0};
        switches[SW_BLOCK_CR_TURBO] = new int[]{23, 11, 11, 3};
        switches[SW_BLOCK_HX_TURBO] = new int[]{22, 10, 11, 0};
    }
    private static final Map<Integer, int[]> switchStates = new HashMap<>();
    static {
        switchStates.put(SW_OPEN_VCR00, new int[]{15, 0, 1});  // Read area, open bit, shut bit
        switchStates.put(SW_OPEN_VHX00, new int[]{15, 2, 3});
    }
    private static final int[] buttons = new int[Buttons.NUM_BUTTONS];
    static {
        buttons[Buttons.BTN_BLOCK_HVSTAT] = 20;
        buttons[Buttons.BTN_BLOCK_CVSTAT] = 21;
        buttons[Buttons.BTN_BLOCK_HVPUMP] = 22;
        buttons[Buttons.BTN_BLOCK_CVPUMP] = 23;
    }
    private static final int[][] latches = new int[Latches.NUM_LATCHES][];
    static {
        latches[Latches.LATCH_CR_VACUUM] = new int[]{1, 14, 3, 18, 0};   // Reset bit, on area, on bit,
        latches[Latches.LATCH_CR_GATE_AO] = new int[]{9, 15, 4, 18, 2};  //    ind area, ind bit
        latches[Latches.LATCH_CR_GATE_NFC] = new int[]{8, 10, 3, 18, 1};
        latches[Latches.LATCH_CR_PUMP] = new int[]{11, 11, 3, 18, 3};
        latches[Latches.LATCH_HX_VACUUM] = new int[]{0, 14, 0, 18, 4};
        latches[Latches.LATCH_HX_GATE_AO] = new int[]{5, 15, 5, 18, 6};
        latches[Latches.LATCH_HX_GATE_NFC] = new int[]{4, 9, 3, 18, 5};
        latches[Latches.LATCH_HX_PUMP] = new int[]{10, 11, 0, 18, 7};
    }
    private static final int[][] conditions = new int[Conditions.NUM_CONDITIONS][];
    static {
        conditions[Conditions.COND_CR_FORELINE_VAC] = new int[]{11, 6};  // Area, bit
        conditions[Conditions.COND_CR_TRB_PRESS_10] = new int[]{16, 5};
        conditions[Conditions.COND_CR_TRB_PUMP_OFF] = new int[]{16, 6};
        conditions[Conditions.COND_CR_VACUUM_001] = new int[]{17, 5};
        conditions[Conditions.COND_CR_VACUUM_01] = new int[]{17, 4};
        conditions[Conditions.COND_HX_FORELINE_VAC] = new int[]{11, 7};
        conditions[Conditions.COND_HX_TRB_PRESS_10] = new int[]{16, 3};
        conditions[Conditions.COND_HX_TURBO_OFF] = new int[]{16, 4};
        conditions[Conditions.COND_HX_VACUUM_001] = new int[]{17, 7};
        conditions[Conditions.COND_HX_VACUUM_01] = new int[]{17, 6};
    }


    /**
     *   Constructor.
     */
    public VacPlutoDevice()
    {
        super(NUM_AREAS);
    }


    /**
     *  Performs basic initialization.
     */
    @Override
    protected void initDevice()
    {
        super.initDevice();
    }


    /**
     *  Gets the switch device ID.
     * 
     *  @return  The device ID
     */
    @Override
    public int getSwitchDevice()
    {
        return Devices.DEVC_PLUTO;
    }


    /**
     *  Sets a switch on or off.
     *
     *  For the vacuum Pluto, this is implemented as a pair of push buttons,
     *  one for on, one for off.
     *
     *  @param  sw  The switch number.
     *  @param  on  The on state to set: true or false
     */
    @Override
    public void setSwitch(int sw, boolean on)
    {
        int bitNum = switches[sw][on ? SWDI_ON_BIT : SWDI_OFF_BIT];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets the on state of a switch.
     *
     *  The state is not the state of the bit that was toggled, but is read back
     *  from the PLC output line.
     *
     *  @param  sw  The switch number.
     *  @return  Whether the switch is on
     */
    @Override
    public Boolean isSwitchOn(int sw)
    {
        int[] swData = switches[sw];
        Integer value = readAddBit(swData[SWDI_READ_AREA], swData[SWDI_READ_BIT] + 16);
        return value != null ? value != 0 : null;
    }


    /**
     *  Gets the detailed device state of a switch.
     * 
     *  Used only for gate valves
     * 
     *  @param  sw  The switch number
     *  @return  The device state, or null if not a gate valve
     */
    public DeviceState getSwitchState(int sw)
    {
        int[] stateDesc = switchStates.get(sw);
        if (stateDesc == null) return null;
        Integer openValue = readAddBit(stateDesc[SWSI_READ_AREA], stateDesc[SWSI_OPEN_BIT] + 16);
        Integer shutValue = readAddBit(stateDesc[SWSI_READ_AREA], stateDesc[SWSI_SHUT_BIT] + 16);
        if (openValue == null || shutValue == null) return DeviceState.OFFLINE;
        return openValue == 1 ? shutValue == 0 ? DeviceState.OPEN : DeviceState.ERROR : shutValue == 1 ? DeviceState.SHUT : DeviceState.TRANSIT;
    }


    /**
     *  Presses a button.
     * 
     *  @param  btn  The button number
     */
    public void pressButton(int btn)
    {
        int bitNum = buttons[btn];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets whether a latched condition is active.
     * 
     *  @param  cond  The condition number
     *  @return  Whether active - indicated by the bit being 0
     */
    public Boolean isLatchActive(int cond)
    {
        int[] condData = latches[cond];
        Integer value = readAddBit(condData[LTDI_READ_AREA], condData[LTDI_READ_BIT] + 16);
        return value != null ? value == 0 : null;
    }


    /**
     *  Gets whether a latched condition is latched.
     * 
     *  @param  cond  The condition number
     *  @return  Whether latched - indicated by the bit being 1
     */
    public Boolean isLatchLatched(int cond)
    {
        int[] condData = latches[cond];
        Integer value = readAddBit(condData[LTDI_PEND_AREA], condData[LTDI_PEND_BIT] + 16);
        return value != null ? value != 0 : null;
    }


    /**
     *  Clears a latched condition.
     * 
     *  @param  cond  The condition number
     */
    public void clearLatch(int cond)
    {
        int bitNum = latches[cond][LTDI_RESET_BIT];
        toggleBit(bitNum / 16, bitNum & 0x0f);
    }


    /**
     *  Gets whether a condition is active.
     * 
     *  @param  cond  The condition number
     *  @return  Whether active - indicated by the bit being 1
     */
    public Boolean isConditionActive(int cond)
    {
        int[] condData = conditions[cond];
        Integer value = readAddBit(condData[CNDI_READ_AREA], condData[CNDI_READ_BIT] + 16);
        return value != null ? value != 0 : null;
    }

}
